@@ -1,9 +1,9 @@
 # coding=utf-8
 #
 # This file is part of Hypothesis, which may be found at
-# https://github.com/HypothesisWorks/hypothesis-python
+# https://github.com/HypothesisWorks/hypothesis/
 #
-# Most of this work is copyright (C) 2013-2018 David R. MacIver
+# Most of this work is copyright (C) 2013-2019 David R. MacIver
 # (david@drmaciver.com), but it contains contributions by others. See
 # CONTRIBUTING.rst for a full list of people who may hold copyright, and
 # consult the git log if you need to determine who owns an individual
@@ -11,7 +11,7 @@
 #
 # This Source Code Form is subject to the terms of the Mozilla Public License,
 # v. 2.0. If a copy of the MPL was not distributed with this file, You can
-# obtain one at http://mozilla.org/MPL/2.0/.
+# obtain one at https://mozilla.org/MPL/2.0/.
 #
 # END HEADER
 
@@ -22,7 +22,8 @@ from distutils.version import LooseVersion
 import pytest
 
 from hypothesis import Verbosity, core, settings
-from hypothesis._settings import note_deprecation
+from hypothesis._settings import Verbosity, note_deprecation
+from hypothesis.errors import InvalidArgument
 from hypothesis.internal.compat import OrderedDict, text_type
 from hypothesis.internal.detection import is_hypothesis_test
 from hypothesis.reporting import default as default_reporter, with_reporter
@@ -78,7 +79,8 @@ def pytest_report_header(config):
     settings_str = settings.get_profile(profile).show_changed()
     if settings_str != "":
         settings_str = " -> %s" % (settings_str)
-    return "hypothesis profile %r%s" % (profile, settings_str)
+    if config.option.verbose >= 1 or settings.default.verbosity >= Verbosity.verbose:
+        return "hypothesis profile %r%s" % (profile, settings_str)
 
 
 def pytest_configure(config):
@@ -110,11 +112,25 @@ def pytest_configure(config):
 gathered_statistics = OrderedDict()  # type: dict
 
 
-@pytest.mark.hookwrapper
+@pytest.hookimpl(hookwrapper=True)
 def pytest_runtest_call(item):
-    if not (hasattr(item, "obj") and is_hypothesis_test(item.obj)):
+    if not hasattr(item, "obj"):
+        yield
+    elif not is_hypothesis_test(item.obj):
+        # If @given was not applied, check whether other hypothesis decorators
+        # were applied, and fail if so
+        if getattr(item.obj, "_hypothesis_internal_settings_applied", False):
+            raise InvalidArgument(
+                "Using `@settings` on a test without `@given` is completely pointless."
+            )
         yield
     else:
+        if item.get_closest_marker("parametrize") is not None:
+            # Give every parametrized test invocation a unique database key
+            item.obj.hypothesis.inner_test._hypothesis_internal_add_digest = item.nodeid.encode(
+                "utf-8"
+            )
+
         store = StoringReporter(item.config)
 
         def note_statistics(stats):
@@ -129,7 +145,7 @@ def pytest_runtest_call(item):
             item.hypothesis_report_information = list(store.results)
 
 
-@pytest.mark.hookwrapper
+@pytest.hookimpl(hookwrapper=True)
 def pytest_runtest_makereport(item, call):
     report = (yield).get_result()
     if hasattr(item, "hypothesis_report_information"):
@@ -186,9 +202,10 @@ def pytest_collection_modifyitems(items):
                 note_deprecation(
                     "%s is a function that returns a Hypothesis strategy, "
                     "but pytest has collected it as a test function.  This "
-                    "is useless as the function body will never be executed."
+                    "is useless as the function body will never be executed.  "
                     "To define a test function, use @given instead of "
-                    "@composite." % (item.nodeid,)
+                    "@composite." % (item.nodeid,),
+                    since="2018-11-02",
                 )
 
             item.obj = note_strategy_is_not_test
